###########
# this function will provide the same functionality as choosing
# the Open Data menu item from the File menu on the control window
# 
# will need one parameter - the name of the data set to load
###########
loadData<-function(dataName)
{
  if (dataName != "")
  {
    # added for R-2.0.0
    # now need to break into 2 pieces if there are parentheses
    splitCurData<-unlist(strsplit(dataName, " "))

    if (length(splitCurData) > 1)
    {
      # need to remove parentheses from second element
      removeParen<-splitCurData[2]
      removeParen<-substr(removeParen, 2, nchar(removeParen)-1)
      dataParem<-removeParen
      newDataName<-splitCurData[1]
    }
    else
    {
      dataParem<-splitCurData[1]
      newDataName<-splitCurData[1]
    }

    data(list=dataParem)

    # now check to make sure it has been loaded
    if (!is.na(match(newDataName, ls(.GlobalEnv))))    
    {
      # only load dataframes or matrices
      if (is.data.frame(eval(as.name(newDataName))) ||
          is.matrix(eval(as.name(newDataName))) )
      {
         dMessage<-new("gAddDataMessage", 
                       data=list(data=eval(as.name(newDataName))),
                       dataName=newDataName)
         handleMessage(dMessage)

        cat("The", newDataName, "data set has been loaded.", "\n")
      }
      else
      {
        
        cat("The", newDataName, "data set could not be loaded.", "\n")
      }
    }
  }
}

###########
# this function will provide the same functionality as choosing
# the Open File menu item from the File menu on the control window
# 
# will need one parameter - the path of the file to load
###########
loadFile<-function(filePath)
{
  # need to determine the file extension, which will be the second
  # entry after splitting on "."
  fileSplitExt<-unlist(strsplit(filePath, "\\."))
  fileExt<-fileSplitExt[length(fileSplitExt)]

  fileSplitName<-unlist(strsplit(filePath, "\\/"))
  fileName<-fileSplitName[length(fileSplitName)]
  # need to remove the extension from fileName
  fileName<-unlist(strsplit(fileName, "\\."))[1]

  upFileExt<-toupper(fileExt)

  newData<-switch(upFileExt,
        "R"=source(file=filePath),
        "RDA"=load(file=filePath, envir=.GlobalEnv),
        "TAB"=read.table(file=filePath, header=TRUE),
        "CSV"=read.table(file=filePath, header=TRUE,sep=";"),
        "RDATA"=load(file=filePath, envir=.GlobalEnv),
        "TXT"=read.table(file=filePath, header=TRUE),
        cat(fileName, "does not have a recognized file extension.", "\n")
  )

  # now need to decide whether to load newData
  if (upFileExt == "R" || upFileExt == "RDA" || upFileExt == "TAB" ||
      upFileExt == "CSV" || upFileExt == "RDATA")
    loadDF(newData, upFileExt, fileName)
}

###########
# this function will provide the same functionality as choosing
# the View Data menu item from the Display menu on the control window
# 
# can only create view of the active MVC
###########
createSpreadsheet<-function()
{
  activeMVC<-get("activeMVC", mvcEnv)
  if (activeMVC != "")
  {
    # create and handle a message to create a view
    vMessage<-new("gAddViewMessage", dataName=activeMVC, type="spreadView", 
                mData=list())
    handleMessage(vMessage)
  }
  else
    print("There is no active MVC.  You must load some data.")
}

###########
# this function will provide the same functionality as choosing
# the Plot Data menu item from the Display menu on the control window
# 
# can only create plot of the active MVC
#
# will need several parameters - 
#  the variable (column) name for the x axis
#  the variable (column) name for the y axis
#  the row names (if the user only wants to plot a subset)
###########
createSPlot<-function(varx, vary, rowNames="")
{
  activeMVC<-get("activeMVC", mvcEnv)
  if (activeMVC != "")
  {
    curData<-getData(name=activeMVC)
    varNames<-colnames(curData)
    numRows<-dim(curData)[1]
    Xindex<-match(varx, varNames)
    Yindex<-match(vary, varNames)
    if (rowNames == "")
      DFrows<-rownames(curData)
    else
      DFrows<-rowNames
    DFcolumns<-c(Xindex, Yindex)

    # create a message to create a view
    vMessage<-new("gAddViewMessage", dataName=activeMVC, type="sPlotView",
            mData=list(dfRows=DFrows, xvar=varx, yvar=vary))
    handleMessage(vMessage)
  }
  else
    print("There is no active MVC.  You must load some data.")
}

###########
# this function will provide the same functionality as choosing
# one of the menu items from the Event menu (ex. choosing Set Mouse Over,
# Set Left Button Click, etc.)
#
# will need two parameters:
#  event will be an item from "possEvents" in mvcEnv (right now the options
#    are mouseOver, leftButtonClick, middleButtonClick, and rightButtonClick)
#  shortName will be a character string referring to the shortName slot from
#    an object of class gEventFun (i.e. shortName is a description of the 
#    callback function) 
#  if shortName is left blank, then that means that there will be no active
#    active callback function for the event
###########
setCallFunc<-function(event, shortName="")
{
  if (event %in% get("possEvents", mvcEnv))
  {
    aMVC<-get("activeMVC", mvcEnv)
    if (aMVC != "")
    { 
      curMVC<-getMVC(aMVC)
      controlEnv<-controller(curMVC)

      # now need to make sure that callName is a valid value
      curEvents<-get(event, controlEnv)
      curShorts<-unlist(lapply(curEvents, function(x) {shortName(x$eventFun)}))
      if (shortName %in% curShorts || shortName=="")
      {
        # first need to remove any old active function
        activeFun<-unlist(lapply(curEvents, function(y)
                            {y$assigned==TRUE}))
        if (any(activeFun))
        {
          curActiveIndex<-which(activeFun)
          curEvents[[curActiveIndex]][["assigned"]]<-FALSE
        }              

        # then assign this callback function to be active
        if (shortName != "")
        {
          curIndex<-match(shortName, curShorts)
          curEvents[[curIndex]][["assigned"]]<-TRUE
        }
        assign(event, curEvents, controlEnv)

        # need to call the preprocessing function
        if (shortName != "")
        {
          preprocess<-preprocessFun(curEvents[[curIndex]][["eventFun"]])
          if (length(preprocess) > 0)
            for (i in 1:length(preprocess))
            {
              do.call(preprocess[[i]], list())
            }
        }
        if (shortName != "")
          print(paste(shortName, "has been set as the active function for", 
                      event))
        else
          print(paste("No action is active for", event))
      }
      else
      {
        print("The description of the callback function you gave as a parameter is not a valid value.")
      }
    }
  }
  else
  {
    print("Please use the function, getEvents, to find valid values for the event parameter")
  }
}

############
# this function will return all of the possible descriptions
# (shortName slot) for callback functions for this event
#
# event will be an item from "possEvents" in mvcEnv (right now the options
#   are mouseOver, leftButtonClick, middleButtonClick, and rightButtonClick)
############
getDescForEvent<-function(event)
{
  if (event %in% get("possEvents", mvcEnv))
  {
    aMVC<-get("activeMVC", mvcEnv)
    if (aMVC != "")
    {
      curMVC<-getMVC(aMVC)
      controlEnv<-controller(curMVC)
      curEvents<-get(event, controlEnv)
      
      if (length(curEvents) > 0)
      {
        curDesc<-unlist(lapply(curEvents, function(x) {shortName(x$eventFun)}))

        return(curDesc)
      }
      else
        print("There are currently no callback functions for this event.")
    }
  }
  else
  {
    print("Please use the function, getEvents, to find valid values for the event parameter")
  }
}

#############
# this function will return all possible events that can be linked to
# callback functions (so it will return the values from "possEvents" in
# mvcEnv
#############
getEvents<-function()
{
  return(get("possEvents", mvcEnv))
}




###########
# this function will provide the same functionality as choosing
# the Find menu item from the Edit menu on the control window
# 
# will need several parameters - 
#  matchOn will decide whether to find based on Row or Data
#  value will be the value to find
###########
findElement<-function(value, matchOn="Row")
{
  # first need to check if there is at least one view 
  if (getNumberofViews() > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)
   
    # find if there is anything attached to the left button click event
    lbc<-get("leftButtonClick", controlEnv)
    # see if any of the functions are active
    activeLBCFun<-unlist(lapply(lbc, function(y)
        {y$assigned==TRUE}))
    curIndex<-which(activeLBCFun)
    if (length(curIndex)==1)
    {    
      booFind<-TRUE
      curEventFun<-lbc[[curIndex]][["eventFun"]]
      actualFun<-callFun(curEventFun)

      curDF<-modelData(model(curMVC))
      rowNames<-rownames(curDF)
      actualData<-curDF

      if (matchOn=="Row" && value %in% rowNames)
      {
        do.call(actualFun, list(value))
      }
      if (matchOn=="Data")
      {
        # check if the curText is a data element
        matchVals<-apply(actualData, 2, function(x)
           {
             # to get rid of leading white spaces
             x <- gsub("^[[:space:]]+", "", x)
             # to get rid of tailing white spaces
             x <- gsub("[[:space:]]+$", "", x)
             match(value, x)
            })
        if (!all(is.na(matchVals)))
        {
          minMatch<-min(matchVals, na.rm=TRUE)
          curRow<-rowNames[minMatch]

          do.call(actualFun, list(curRow))
        }        
      }
    }
    else
      print("There must be a callback function linked to the left button click event")
  }
  else
    print("You must have at least one view created to perform find.")
}

###########
# this function will provide the same functionality as choosing
# the Replot menu item from the Edit menu on the control window
###########
replotViews<-function()
{
  if (getNumberofViews() > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    # need to replot all these views
    curviewList<-viewList(getMVC(activeMVC))

    if (length(curviewList) > 0)
    {
      uvMessage<-new("gUpdateViewMessage", type="redrawView",
                         mData=list(), dataName=activeMVC)
      handleMessage(uvMessage)      
    }
  }
  else
    print("There is no view to replot.")
}

###########
# this function will provide the same functionality as choosing
# the Reset menu item from the Edit menu on the control window
###########
resetData<-function()
{
  if (getNumberofViews() > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    # need to create an update data message
    # type="reset"
    dfMessage<-new("gUpdateDataMessage", type="reset", mData=list(),
                       dataName=activeMVC, from="")
    handleMessage(dfMessage)
  }
  else
    print("There must be an open view to reset the data.")
}

##########
# changed 10/24/05 to match mvcEnv
#
# need a function that creates a mvcEnv variable that will create the
# main menu items and sets the keyVals variable in mvcEnv
#
# this will need to be loaded in the onLoad function
#
# this is needed because we need the keyVals variable to be set even if 
# the user does not open the control window (so things can be done from
# the command line only)
#
# any key vals that need to be used from views created from the command line
# need to be set in this function - see the example for quit (need to call
# the function addToKeyVals)
##########
createMenuItemVariables<-function()
{
  # create a mvcEnv variable for each menu 
  fileItems<-list()
  fileItems[[1]]<-list(text="_Open File  Ctrl+O", action="openf")
  fileItems[[2]]<-list(text="Open _Data  Ctrl+D", action="opend")
  fileItems[[3]]<-list(text="_Quit       Ctrl+Q", action="quit")
  assign("fileItems", fileItems, mvcEnv)

  # add a key press event for the Quit command even if the 
  # control window is not created
  addToKeyVals(text="_Quit       Ctrl+Q", action="quit")

  editItems<-list()
  editItems[[1]]<-list(text="_Find             Ctrl+F", action="find")
  editItems[[2]]<-list(text="_Replot           Ctrl+R", action="replot")
  editItems[[3]]<-list(text="Rese_t            Ctrl+T", action="reset")
  editItems[[4]]<-list(text="Set _Active MVC   Ctrl+A", action="setActiveMVCGUI")
  editItems[[5]]<-list(text="D_elete MVC       Ctrl+E", action="deleteMVCGUI")
  assign("editItems", editItems, mvcEnv)

  displayItems<-list()
  displayItems[[1]]<-list(text="_View Data      Ctrl+V", action="viewData")
  displayItems[[2]]<-list(text="_Plot Data      Ctrl+P", action="plotD")
  assign("displayItems", displayItems, mvcEnv)

  eventItems<-list()
  eventItems[[1]]<-list(text="_Set Mouse Over          Ctrl+S", 
			action="setMouseOver")
  eventItems[[2]]<-list(text="Set Left _Button Click   Ctrl+B", 
			action="setLeftBC")
  eventItems[[3]]<-list(text="Set _Middle Button Click Ctrl+M",
                        action="setMiddleBC")
  eventItems[[4]]<-list(text="Set R_ight Button Click  Ctrl+I", 
			action="setRightBC")
  assign("eventItems", eventItems, mvcEnv)
}

##########
# changed 10/24/05 to match mvcEnv
#
# This function will add to the keyVals variable stored in mvcEnv.
# It is this variable, keyVals, that determines what key press events
# the Gtk windows will respond to.  Thus, the keyVals variable will be 
# different if the views are created through the command line functions or
# through the user interface.  
#
# If the control window is created, then there will be key press events for
# all menu items (this is done through two function calls, including
# createMenuItem and createView).
# If the views are created through the command line functions, then there
# will be key press events for quitting and for setting the focus on a view
# (this is done through the functions createMenuItemVariables and createView).
#
# text is the text that would be placed on the menu item label
# action is the response that should occur when the key press event occurs
# (may either be a function name or the text used in updateControlWindow)
# ... is only used when the action corresponds to focusing on a view (so
# ... will contain the window number of the view)
##########
addToKeyVals<-function(text, action, ...)
{
  newLabel<-gtkAccelLabelNew(text)
  newKey<-gtkLabelParseUline(newLabel, text)

  # get the accelerator character
  labelSplit<-unlist(strsplit(text,"_"))
  if (length(labelSplit) > 1)
    accelChar<-substr(labelSplit[2],1,1)
  else
    accelChar<-""

  keyVals<-get("keyVals", mvcEnv)
  if (length(keyVals) > 0)
  {
    # need to check that this key val hasn't already been added
    # test all three characteristics (keyval, action, and accelChar)
    booMatch<-FALSE
    for (i in 1:length(keyVals))
    {
      if (keyVals[[i]]$keyval==newKey & keyVals[[i]]$action==action &
          keyVals[[i]]$accelChar==accelChar)
        booMatch<-TRUE
    }
    # only add if this is not already in keyVals variable
    if (!booMatch)
    {
      lenKV<-length(keyVals)
      keyVals[[lenKV+1]]<-list(keyval=newKey, action=action,
                           accelChar=accelChar, ...)
    }
  }
  else
  {
    lenKV<-length(keyVals)
    keyVals[[lenKV+1]]<-list(keyval=newKey, action=action,
                           accelChar=accelChar, ...)
  }
  assign("keyVals", keyVals, mvcEnv)
}

###########
# a command line function that performs the same functionality as
# the Quit menu item (under the File menu)
###########
quitiSPlot<-function()
{
  clearAllWindows()
}

############
# a command line function that performs the same functionality as
# the Set Active MVC menu item (under the Edit menu)
#
# the one parameter is the name of the MVC that should be active
############
setActiveMVC<-function(aMVC)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    mNames<-getModelNames()
    if (aMVC %in% mNames)
    {
      assign("activeMVC", aMVC, mvcEnv)
    }
    else
      print(paste(aMVC, "is not the name of a loaded MVC."))
  }
  else
    print("There are no MVCs loaded.")
}

##########
# a command line function that performs the same functionality as
# the Delete MVC menu item (under the Edit menu) - note that
# the active MVC cannot be deleted
#
# the one parameter is the name of the MVC that should be deleted
##########
deleteMVC<-function(dMVC)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    mNames<-getModelNames()
    if (dMVC %in% mNames)
    {
      activeMVC<-get("activeMVC", mvcEnv)
      if (dMVC != activeMVC)
      {
        # now need to remove the views for this MVC 
        delMVC<-getMVC(dMVC)
        vList<-viewList(delMVC)
        
        if (length(vList) > 0)
          for (i in 1:length(vList))
            setDeleteEvents(vList[[i]])        

        # then remove the MVC from MVCList in mvcEnv
        # find the index of the item to remove    
        allNames<-getModelNames(sort=FALSE)
        index<-match(dMVC, allNames)

        newMVCList<-MVCList[-index]
        assign("MVCList", newMVCList, mvcEnv)
      }
      else
        print("The active MVC cannot be deleted.")
    }
    else
      print(paste(dMVC, "is not the name of a loaded MVC."))
  }
  else
    print("There is no MVC loaded to delete.")
}

##########
# this function will replace the createColorBrowser function when
# the user is not using the GUI, but wants to color points as a callback
# function (see functions createColorBrowser and colorPoint)
#
# newColor is the character string describing the new color 
##########
setColor<-function(newColor)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)
    
    assign("currentColor", newColor, controlEnv)
  }
}

############
# this function will replace the setTooltipsGUI function when the user
# is not using the GUI, but wants tooltips as a callback function (see
# functions tooltips and setTooltipsGUI)
#
# name is the name of the column for tooltips or "rowNames" if the user wants
# the row names as the tooltips
############
setTooltips<-function(name)
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)
    curData<-modelData(model(curMVC))

    colNames<-colnames(curData)
    if (name %in% c(colNames, "rowNames"))
    {
      rNames<-rownames(curData)

      if (name=="rowNames")
      {
        # set the tooltips to the row name
        ttVec<-rNames
        names(ttVec)<-rNames
        assign("ttToShow", ttVec, controlEnv)        
      }
      else
      {
        cNames<-colnames(curData)
        ttVec<-curData[,match(name, cNames)]
        names(ttVec)<-rNames
        assign("ttToShow", ttVec, controlEnv)        
      }
    }
    else
      print(paste("The parameter must either be a column name from the", 
                 "active data set or it must be 'rowNames' to indicate",
                 " the row names."))    
  }
}

#############
# a function to return the active MVC object name
#############
getActiveMVC<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    return(activeMVC)
  }
  else
    print("There is no active MVC.")
}

############
# a function to return the MVCList variable in mvcEnv
# this lets the user see all of the MVC objects currently loaded
############
getMVCList<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  return(MVCList)
}

##########
# a function to return the current color stored in the active MVC's controller 
##########
getColor<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    controlEnv<-controller(getMVC(activeMVC))
    curColor<-get("currentColor", controlEnv)
    return(curColor)
  }
  else
    print("There is no active MVC.")
}

##########
# a function to return the current tooltips stored in the active MVC's 
# controller 
##########
getTooltips<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    controlEnv<-controller(getMVC(activeMVC))
    curTooltips<-get("ttToShow", controlEnv)
    return(curTooltips)
  }
  else
    print("There is no active MVC.")
}

############
# want to add a callback function from the command line
# will need to be added for 2 situations: 1. for MVC objects that are 
#  already loaded and 2. for MVC objects that will be added in the 
#  future
# callFunction is the actual callback function
# preprocessing function is list of functions or NULL
############
addCBFunction<-function(callFunction, shortName, preprocessingFun, 
                              eventLists)
{
  MVCList<-get("MVCList", mvcEnv)
   # then need to add this new event to each loaded MVC object
  if (length(MVCList) > 0)
  {
    for (i in 1:length(MVCList))
    {
      addCallEvent(callFunction, shortName, preprocessingFun, eventLists, 
                       MVCList[[i]])
    }
  }
  # add to a mvcEnv variable that createDefaultCallEvents will check when a
  # new MVC object is loaded
  # the mvcEnv variable is called newCallFuns
  newCallFuns<-get("newCallFuns", mvcEnv)
  newCallFuns[[length(newCallFuns)+1]]<-list(callFunction=callFunction,
		shortName=shortName, preprocessingFun=preprocessingFun,
		eventLists=eventLists)
  assign("newCallFuns", newCallFuns, mvcEnv)
}
