
##########
# added 10/25/05 to see if there are any views
##########
getNumberofViews<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    numViews<-unlist(lapply(MVCList, function(x) {
                   length(viewList(x))
                   }))
    totalnumViews<-sum(numViews)
  }
  else 
    totalnumViews<-0

  return(totalnumViews)
}

#################
# get values from the environments through the command line
#################
getMVCEnvVar<-function(varName)
{
  return(get(varName, mvcEnv))
}

#################
# setParamList is a function to set the parameter list for making a 
# scatterplot
# dataF is the dataframe and dfRows are the dataframe rows
#################
setParamList<-function(dataF, dfRows, plotObject)
{
  colIndex<-match("color", names(dataF))
  curColors<-dataF[dfRows, colIndex]
  pchIndex<-match("pch", names(dataF))
  curPch<-dataF[dfRows, pchIndex]
  hideIndex<-match("hide", names(dataF))
  curHide<-dataF[dfRows, hideIndex]
  highIndex<-match("highlit", names(dataF))
  curHigh<-dataF[dfRows, highIndex]
  
  # make the points that are hidden be the background color
  plotBG<-plotPar(plotObject)$bg
  curColors[curHide]<-plotBG

  return(list(col=curColors, pch=curPch))
}

##################
# identifyPoint determines which point on the scatterplot
# the user clicked (or NULL if no point was clicked)
#
# curplot is the plot information (one element from viewList)
# xyloc is the x and y location that was clicked
##################
identifyPoint<-function(curplot, xyloc)
{
#  print(xyloc)
  # get the cxy of the active plot
  cxy<-par("cxy")
  pointWidth<-cxy[1]
  pointHeight<-cxy[2]  

  plotDf<-dataName(curplot)
  plotRows<-dfRows(curplot)
  plotX<-xvar(curplot)
  plotY<-yvar(curplot)

  # need to get the data from the model
  mvcList<-get("MVCList", mvcEnv)

  curData<-getData(name=plotDf)
  rowIndices<-match(plotRows, rownames(curData))
  colxIndex<-match(plotX, colnames(curData))
  colyIndex<-match(plotY, colnames(curData))

  x<-curData[rowIndices, colxIndex]
  y<-curData[rowIndices, colyIndex]

  # check if x or y is a factor
  if (is(x, "factor"))
    x<-as.numeric(x)
  if (is(y, "factor"))
    y<-as.numeric(y)

  rowNames<-row.names(curData)

  # now need to identify the closest point
  totdiff<-(xyloc$x-x)^2+(xyloc$y-y)^2
  booClosest<-min(totdiff)==totdiff
  
  closestPoint<-rowNames[booClosest]
  closestXY<-list(x=x[booClosest], y=y[booClosest])
#  print(closestXY)

  # need to ensure that the closestPoint was actually clicked
  # use cxy
  x.within<-FALSE
  y.within<-FALSE

  if ((xyloc$x<=closestXY$x) && (xyloc$x>=closestXY$x-(pointWidth/2)))
      x.within<-TRUE
  if ((xyloc$x>closestXY$x) && (xyloc$x<=closestXY$x+(pointWidth/2)))
      x.within<-TRUE  

  if ((xyloc$y<=closestXY$y) && (xyloc$y>=closestXY$y-(pointHeight/2)))
      y.within<-TRUE
  if ((xyloc$y>closestXY$y) && (xyloc$y<=closestXY$y+(pointHeight/2)))
      y.within<-TRUE 

  if (x.within && y.within)
    return(list(closestXY=closestXY, closestPoint=closestPoint))
  else
    return(NULL)
}

#############
# remove the devices from R's device list
#############
removeDevices<-function(curDev)
{
  # remove the devices from R's device list
  sapply(curDev, function(x){dev.off(x)}, USE.NAMES=FALSE)
}

################
# getPlotDev makes a vector of the device element in the viewList
# must make sure that viewList contains only elements of class plotView
################
getPlotDev<-function(viewList)
{
  dev.ele<-lapply(viewList, function(x)
                           {
                             if (is(x, "plotView"))
                               return(plotDevice(x))
                             else
                               return(0)
                           })
  dev.ele<-unlist(dev.ele)
  return(dev.ele)
}

################
# addDrawingArea adds a drawing area to a gtk widget
#
# wid is the widget, dr is the drawing area,
# width is the drawing area width, height is the drawing area height,
# devVec is the device vector for the gtk window that will contain this
# drawing area,
# addType can be either 1,2,3,4 or 5 depending on which widget the drawing
# area is being added to (1=window,2=2nd pane,3=notebook,4=scrolled window,5=hbox or vbox)
# lab is the label, devName is the device name
################
addDrawingArea<-function(wid, dr, width, height, devVec, addType=1,
                         lab="", devName="")
{
  # set up the drawing area
  asGtkDevice(dr)
  dr$SetUsize(width, height)

  # 1 is for normal addition to a window, 
  # 2 is for addition to vertical or horizontal pane (2nd pane),
  # 3 is for addition to a notebook, 4 is for addition to a scrolled window
  # 5 for adding to box (horizontal or vertical)
  switch(addType,
         wid$Add(dr),
         wid$Add2(dr),
         wid$AppendPage(dr, gtkLabel(lab)),
         wid$AddWithViewport(dr),
         gtkBoxPackStart(wid, dr))

  # add to the device vector for the widget
  devVec<-dev.cur()
  
  # return the updated device vector
  return(devVec)
}

################
# removeEnvPlot removes plots from viewList based on the
# device number
# plotItem is the plot object
################
removeEnvPlot<-function(plotItem)
{
  isPlot<-is(plotItem, "plotView")
  isSpread<-is(plotItem, "spreadView")

  dataName<-dataName(plotItem)
  win<-win(plotItem)

  MVCList<-get("MVCList", mvcEnv)
  allNames<-getModelNames(sort=FALSE)
  mvcindex<-match(dataName, allNames)
  curMVC<-getMVC(dataName)
  curviewList<-viewList(curMVC)

  if (isPlot)
  {
    devVec<-plotDevice(plotItem)

    # need to find the indices of the plots to remove
    dev.num<-getPlotDev(curviewList)

    # sort will remove any NAs and need to make the list decreasing
    # otherwise, the indices won't work because as elements are 
    # removed from the list, the list length decreases (so upper
    # elements have their indices change)
    indicesToRemove<-sort(match(devVec, dev.num), decreasing=TRUE)

    if (length(curviewList)>0 && length(indicesToRemove)>0)
      curviewList[indicesToRemove]<-c()
  }
  if (isSpread)
  {
    booSpread<-unlist(lapply(curviewList, function(x) {is(x, "spreadView")}))
    if (any(booSpread))
    {
      indicesToRemove<-sort(which(booSpread), decreasing=TRUE)
      curviewList[indicesToRemove]<-c()
    }
  }
  # update the plot list
  viewList(curMVC)<-curviewList
  # need to reassign this MVC to MVCList
  MVCList[[mvcindex]]<-curMVC
  assign("MVCList", MVCList, mvcEnv)
}

################
# setDeleteEvents removes views from the viewList
# viewItem is an object of type view
################
setDeleteEvents<-function(viewItem)
{
  # remove from R's device list
  if (is(viewItem, "plotView"))
  {
    removeDevices(plotDevice(viewItem))
  }
    
  # need to also remove views from viewList
  removeEnvPlot(viewItem)

  # only do this if the control window is open
  # need to remove the menu item from the menu
  if (length(get("controlWin", mvcEnv)) > 0)
  { 
    gtkMenuItemRemoveSubmenu(get("windowItem", mvcEnv))
    # need to remove the accelerators from keyVal in mvcEnv
    removeAccelerators()
  
    # now add back the window menu to the control window
    addWindowMenu()
  }
  else
  {
    removeAccelerators()

    # need to add to the window menu for the plots
    MVCList<-get("MVCList", mvcEnv)
    if (length(MVCList) > 0)
    {
      allViews<-lapply(MVCList, viewList)
      for (i in 1:length(allViews))
      {
        curViewList<-allViews[[i]]
        if (length(curViewList) > 0)
        {
          allWinNums<-unlist(lapply(curViewList, winNum))
          for (j in 1:length(allWinNums))
          {
            curLabel<-paste("Window _", allWinNums[j],
                    "  Ctrl+", allWinNums[j], sep="")
            addToKeyVals(text=curLabel, action="activateWindow", 
                     winNum=allWinNums[j]) 
          }
        }
      }
    }
  }
  win(viewItem)$Destroy()     
}

#######
# will remove all accelerators that have an element winNum
#######
removeAccelerators<-function()
{
  keyVals<-get("keyVals", mvcEnv)
  for (i in length(keyVals):1)
  {
    if ("winNum" %in% names(keyVals[[i]]))
    {
      keyVals[[i]]<-c()
    }
  }
  assign("keyVals", keyVals, mvcEnv)
}

################
# closeWin adds a callback to the window for the delete
# event, which closes all devices open in the window
################
closeWin<-function(viewItem)
{
  win<-win(viewItem)

  win$AddCallback("delete_event",
    function(x, y)
    {    
      # x is the widget and y is the event type
      setDeleteEvents(viewItem)            
      # 6/1/06 need to set active win num to NULL
      assign("activeWinNum", c(), mvcEnv)
    }
  )
}

##########
# createKeyPressEvent is to add pseudo-accelerators to any 
# gtk window
##########
createKeyPressEvent<-function(w)
{
  # add a key press event to the window so it can seem like the user can use
  # the accelerators from the control window on any active gtk window
  gtkAddCallback(w, "key-press-event",
    function(obj, ev)
    {
      curStr<-gdkEventKeyGetString(ev)
      curState<-gdkEventKeyGetState(ev)
      curKeyval<-gdkEventKeyGetKeyval(ev)

      setKeyValToAction(curStr, curState, curKeyval)
    }
  )
}

##############
# setKeyValToAction uses the controlEnv variable, keyVals, to match
# a key press to the appropriate action
##############
setKeyValToAction<-function(curStr, curState, curKeyval)
{
  keyVals<-get("keyVals", mvcEnv)

  assignedKeyVals<-unlist(lapply(keyVals, function(x){x$keyval}))
  assignedCurActions<-unlist(lapply(keyVals, function(x){x$action}))
  
  # what if the case is upper vs. lower?
  # note: having the Caps lock down changes the state
  # curState=4 when Ctrl has been pressed
  # curState=6 when Caps Lock is on and Ctrl has been pressed
  if (curState == 4 || curState == 6)
  {
    index<-which(assignedKeyVals %in% curKeyval)
    
    # first test if we just have the wrong capitalization
    if (length(index)==0)
    {
      # 32 is the difference between lower and upper caps on the same letter
      index<-which(assignedKeyVals %in% (curKeyval+32))
    }

    if (length(index) > 0)
    {
      if (length(index) > 1)
      {
        # need to find the correct index; the one where curAction != ""
        index<-index[assignedCurActions[index] != ""]
      }

      curAction<-assignedCurActions[index]

      # check if there are any extra parameters
      # each keyVals item should have 3 elements: keyval, action, and
      # accelChar so anything after the 3 is an extra parameter 
      if (length(keyVals[[index]]) > 3)
      {
        totalLen<-length(keyVals[[index]])
        updateCurAction(curAction, unlist(keyVals[[index]][4:totalLen]))
      }
      else
      {
        updateCurAction(curAction)
      }
    }
  }
}

#################
# scatterplot is a function to plot a data set
# it is used so that the viewList in a MVC object can be updated
# dfName is the data set name to plot, DFrows are the rows to use
# DFcolumns are the columns to use, dev is the device number
# 
# viewItem is a sPlotView object
#################
scatterplot<-function(viewItem)
{
  dfName<-dataName(viewItem)
  DFrows<-dfRows(viewItem)
  DFcolumns<-c(xvar(viewItem), yvar(viewItem))  
  dev<-plotDevice(viewItem)

  # make sure the length of DFcolumns is 2
  curDF<-getData(name=dfName)
  rowIndeces<-match(DFrows, rownames(curDF))
  colIndeces<-match(DFcolumns, colnames(curDF))
  plotDF<-curDF[rowIndeces, colIndeces]

  dev.set(dev)

  curMVC<-getMVC(dfName)
  curvirtualData<-virtualData(model(curMVC))

  if (length(curvirtualData) > 0)
  {
    # get the highlit column
    curHigh<-curvirtualData[rowIndeces, match("highlit", 
                            names(curvirtualData))]
    curHide<-curvirtualData[rowIndeces, match("hide",
                            names(curvirtualData))]

    # what about the data stored in the DF columns: color, highlit, pch, hide
    paramList<-setParamList(curvirtualData, rowIndeces, viewItem)  
  }

  # it seems like the window needs to be shown before plotting or I
  # get a gdk error
  win<-win(viewItem)
  win$Show()

  # set a few parameters that don't seem to be set correctly for a gtk device
  par("bg"="transparent")
  par("col"="black")
  par("fg"="black")

  # couldn't put in the parameter list directly 
  # note: may have more than one point here and this can cover up the change
  if (length(curvirtualData) > 0)
    plot(plotDF, col=paramList$col, pch=paramList$pch)
  else
    plot(plotDF)

  if (length(curvirtualData) > 0)
  {
    # need to highlight points
    highPoints<-plotDF[curHigh,]
    if (nrow(highPoints) > 0)
    {
      pointCex<-plotPar(viewItem)$cex
      curCex<-as.integer(pointCex)+1
      points(highPoints[,1], highPoints[,2], cex=curCex, 
           col="red")
    }
 
    # need to hide points
    hidePoints<-plotDF[curHide,]
    if (nrow(hidePoints) > 0)
    {
      backgr<-plotPar(viewItem)$bg
      if (backgr=="transparent")
        backgr="white"
      # make sure to fill in the point as background in case it is colored in
      points(hidePoints[,1], hidePoints[,2], col=backgr, pch=19)
    }

    hideAndHigh<-curHigh & curHide
    if (any(hideAndHigh))
    {
      hideAndHighPoints<-plotDF[hideAndHigh,]
      points(hideAndHighPoints[,1], hideAndHighPoints[,2], cex=curCex,
            col=backgr)
    }
  }

  # need to update plot parameter usr
  plotPar(viewItem)<-par(no.readonly=TRUE)

  return(viewItem)
}

################
# createGtkDev is a function to create a Gtk device for plotting
# -also adds a callback for the focus event
################
createGtkDev<-function(w)
{
  dev <- c()

  w$SetUsize(500, 500)

  # need to add rulers
  vbox<-gtkVBox()
  hrul<-gtkHRuler(show=FALSE)
  hrul$SetMetric("pixels")
  hrul$SetRange(0, 100, 0, 10)
  gtkBoxPackStart(vbox, hrul, expand=FALSE)

  hbox<-gtkHBox()
  gtkBoxPackStart(vbox, hbox)
  vrul<-gtkVRuler(show=FALSE)
  vrul$SetMetric("pixels")
  vrul$SetRange(0, 100, 0, 10)
  gtkBoxPackStart(hbox, vrul, expand=FALSE)

  w$Add(vbox)
 
  drArea <- gtkDrawingArea()
  dev<-addDrawingArea(hbox, drArea, 250, 250, dev, devName="Dev1", 
                      addType=5)

  # want to be informed of these events on the device
  drArea$SetEvents(c("pointer-motion-mask", "pointer-motion-hint-mask", 
                   "button-press-mask", "button-release-mask", 
                   "key-press-mask", "key-release-mask"))

  return(list(win=w, drArea=drArea))
}

######
# make an interface for getting data, in case new
# data types are added besides dataframe or matrix
######
getData<-function(name)
{
  if (name %in% getLoadedDF())
  {
    curMVC<-getMVC(name)

    mData<-modelData(model(curMVC))
  }
  else
    mData<-NULL

  return(mData)
}

##########
# sets the focus on a window
##########
activateWindow<-function(winNum)
{
#  print(winNum)
  if (winNum==0)
  {
    controlWin<-get("controlWin", mvcEnv)
    if (length(controlWin) > 0)
    {
#      gtkWindowActivateFocus(controlWin)
      gtkObjectSignalEmit(controlWin, "hide")
      gtkObjectSignalEmit(controlWin, "show")
    }
  }
  else
  {
    MVCList<-get("MVCList", mvcEnv)
    # need to get the window from MVCList
    if (length(MVCList) > 0)
    {
      for (i in 1:length(MVCList))
      {
        curviewList<-viewList(MVCList[[i]])
        curWinNums<-unlist(lapply(curviewList, function(x) {winNum(x)}))
        curWins<-lapply(curviewList, function(x) {win(x)})
        index<-match(winNum, curWinNums)
        if (!is.na(index))
        {
          gtkObjectSignalEmit(curWins[[index]], "hide")
          gtkObjectSignalEmit(curWins[[index]], "show")
        }
      }
    }
  }
}




